"""Keyboard input strategies for different scraper implementations."""

from .base import KeyboardStrategy
from .mtk import MtkKeyboard
from .nppfs import NppfsKeyboard

__all__ = ["KeyboardStrategy", "MtkKeyboard", "NppfsKeyboard"]
