from typing import TypedDict


class CardTransaction(TypedDict):
    """Base interface for card transactions."""

    transacted_at: str
    merchant: str
    amount: str
    installment: str
    status: str
